package pmicrovenus;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.OneToMany;

@Entity
public class Solar extends Casilla implements Serializable {
    
    private String color;
    protected float precioCompra;
    private int nCasas;
    private float precioCasa;
    private float renta0casas;
    float precioHotel;
    float renta1casas;
    float renta2casas;
    float renta3casas;
    float renta4casas;
    float rentaHotel;
    
    @OneToMany(cascade=CascadeType.ALL)
    private List<Solar> asociados;

    public Solar(int id, String des, int px, int py) {
        super(id,des,px,py); 
        precioCompra=300;
        renta0casas=50;
        precioCasa=100;
        nCasas=0;
    }
    
    public Solar(int id, String des, int px, int py, String color, float precioCompra,float renta,float precioCasa) {
        super(id,des,px,py); 
        this.setColor(color);
        this.setPrecioCompra(precioCompra);
        this.setRenta(renta);
        this.setPrecioCasa(precioCasa);
        this.setPrecioHotel(precioCasa*4f);
        asociados = new ArrayList<Solar>();
        super.setValorHipoteca(precioCompra * 0.7f);
 
    }
    public Solar() {
         asociados = new ArrayList<Solar>();
    }
   
  
    @Override
    public float getPrecioCompra() {
        return precioCompra;
    }

    public void setPrecioCompra(float val) {
        this.precioCompra = val;
    }

   
    public float getRenta(int n) {
        
        float res = 0;
        switch (n) {
            case 0: res = renta0casas;break;
            case 1: res = renta1casas; break;
            case 2: res = renta2casas; break;
            case 3: res = renta3casas; break;
            case 4: res = renta4casas; break;
            case 5: res = rentaHotel; break;
        }
        return res;
    }

    public void setRenta(float val) {
        this.renta0casas = val;
        this.renta1casas = val * 1.7f;
        this.renta2casas = val * 2f;
        this.renta3casas = val * 2.3f;
        this.renta4casas = val * 2.6f;
        this.rentaHotel = val * 2.9f;
        
        
    }

    public int getNCasas() {
        return nCasas;
    }

    public void setNCasas(int val) {
        this.nCasas = val;
    }
    
    @Override
    public String getColor() {
        return color;
    }

    public void setColor(String val) {
        this.color = val;
    }
    
    public float getPrecioCasa() {
        return precioCasa;
    }

    public void setPrecioCasa(float precioCasa) {
        this.precioCasa = precioCasa;
    }
    
    public float getPrecioHotel() {
        return precioHotel;
    }

    public void setPrecioHotel(float precioHotel) {
        this.precioHotel = precioHotel;
    }
    
    
    public float getAlquiler() {
        float res = 0;
        switch (getNCasas()) {
            case 0: //Si no tiene construcciones y tiene monopolio y..
                if(this.getPropietario().getMonopolioSolares().toArray().equals(this)){
                    //...si ningun Solar del monopolio esta hipotecado cobras doble
                    boolean doble = true;
                    if(this.isHipotecada()) doble=false;
                    for(int i=0;i<this.getAsociados().size();i++){
                        if (this.getAsociados().get(i).isHipotecada())
                            doble=false;
                    }
                    if (doble)  res = renta0casas * 2;
                    else res = renta0casas;
                }else res = renta0casas; 
                break;
            case 1: res = renta1casas; break;
            case 2: res = renta2casas; break;
            case 3: res = renta3casas; break;
            case 4: res = renta4casas; break;
            case 5: res = rentaHotel; break;
        }
        return res;
    }
    
    
    public void edificar() {
    
        if (getPropietario() == null) return;
        if ((nCasas < 4) && getPropietario().tieneEfectivo(precioCasa))
        {
            nCasas++;
            getPropietario().reintegro(precioCasa);
        }else{
           if ((nCasas == 4) && getPropietario().tieneEfectivo(precioHotel))
            {
                nCasas++;
                getPropietario().reintegro(precioHotel);
            } 
        }
        System.out.println(super.getDescripcion()+" tiene " + nCasas + " casas." );
    }
    
    
    @Override
    public void accion(Tablero t) {
        System.out.println(t.getJugadorActual().getNombre() + " ha caido en la casilla numero " + super.getNum() + ", " + super.getDescripcion());
       // if (isDisponible()==true){
         if (super.getPropietario()==null){
           System.out.println("Comprar solar " + super.getDescripcion() + " por " + getPrecioCompra() + " ? (s/n)"); 
           //Mostrar Dialogo Adquirir Pdad.
           jdlgAdquirirPdad dlg = new jdlgAdquirirPdad(t,t.formulario,true);
           dlg.setLocationRelativeTo(t.formulario);
           dlg.setVisible(true);
        }else{
             if (t.getJugadorActual().tieneSolar(this.getDescripcion())){
                //poner casas?
                 System.out.println("Poner casas?");
            }else{
                if (this.isHipotecada()){
                    
                } else
                    t.getJugadorActual().pagarRentaA(getPropietario(),getAlquiler());
            }
        }
        
    }

    public List<Solar> getAsociados() {
        return asociados;
    }

    public void setAsociados(List<Solar> asociados) {
        this.asociados = asociados;
    }

    @Override
    public void addAsociado(Solar asociado) {
        this.asociados.add(asociado);
    }

}
